@@ -1,5 +1,5 @@
 /************************************************************************************
-  Copyright (C) 2017, 2022, MariaDB Corporation AB
+  Copyright (C) 2017 MariaDB Corporation AB
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -20,12 +20,13 @@
 #define _GNU_SOURCE 1
 #endif
 
-#ifdef HAVE_WINCRYPT
-#undef HAVE_OPENSSL
-#undef HAVE_GNUTLS
+#ifdef _WIN32
+#if !defined(HAVE_OPENSSL)
+#define HAVE_WINCRYPT
+#endif
 #endif
 
-#if defined(HAVE_OPENSSL) || defined(HAVE_WINCRYPT) || defined(HAVE_GNUTLS)
+#if defined(HAVE_OPENSSL) || defined(HAVE_SCHANNEL) || defined(HAVE_GNUTLS)
 
 #include <ma_global.h>
 #include <mysql.h>
@@ -47,11 +48,12 @@
 #include <openssl/err.h>
 #elif defined(HAVE_GNUTLS)
 #include <gnutls/gnutls.h>
-#elif defined(HAVE_WINCRYPT)
+#elif defined(HAVE_SCHANNEL)
 #include <windows.h>
 #include <wincrypt.h>
 #include <bcrypt.h>
-
+#pragma comment(lib, "bcrypt.lib")
+#pragma comment(lib, "crypt32.lib")
 extern BCRYPT_ALG_HANDLE RsaProv;
 extern BCRYPT_ALG_HANDLE Sha256Prov;
 #endif
@@ -84,7 +86,7 @@ static int ma_sha256_scramble(unsigned char *scramble, size_t scramble_len,
   unsigned char digest1[MA_SHA256_HASH_SIZE],
                 digest2[MA_SHA256_HASH_SIZE],
                 new_scramble[MA_SHA256_HASH_SIZE];
-#ifdef HAVE_WINCRYPT
+#ifdef HAVE_SCHANNEL
   MA_HASH_CTX myctx;
   MA_HASH_CTX *ctx= &myctx;
 #else
@@ -92,17 +94,18 @@ static int ma_sha256_scramble(unsigned char *scramble, size_t scramble_len,
 #endif
   size_t i;
 
-  /* check if all specified lengths are valid */
+  /* check if all specified lenghts are valid */
   if (!scramble_len || !source_len || !salt_len)
     return 1;
 
+
   /* Step1: create sha256 from source */
   if (!(ctx= ma_hash_new(MA_HASH_SHA256, ctx)))
     return 1;
   ma_hash_input(ctx, source, source_len);
   ma_hash_result(ctx, digest1);
   ma_hash_free(ctx);
-#ifndef HAVE_WINCRYPT
+#ifndef HAVE_SCHANNEL
   ctx = NULL;
 #endif
 
@@ -112,7 +115,7 @@ static int ma_sha256_scramble(unsigned char *scramble, size_t scramble_len,
   ma_hash_input(ctx, digest1, MA_SHA256_HASH_SIZE);
   ma_hash_result(ctx, digest2);
   ma_hash_free(ctx);
-#ifndef HAVE_WINCRYPT
+#ifndef HAVE_SCHANNEL
   ctx = NULL;
 #endif
 
@@ -132,7 +135,7 @@ static int ma_sha256_scramble(unsigned char *scramble, size_t scramble_len,
 
 /* function prototypes */
 static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql);
-static int auth_caching_sha2_deinit(void);
+static int auth_caching_sha2_deinit();
 static int auth_caching_sha2_init(char *unused1,
     size_t unused2,
     int unused3,
@@ -159,11 +162,11 @@ struct st_mysql_client_plugin_AUTHENTICATION _mysql_client_plugin_declaration_ =
   auth_caching_sha2_client
 };
 
-#ifdef HAVE_WINCRYPT
+#ifdef HAVE_SCHANNEL
 static LPBYTE ma_load_pem(const char *buffer, DWORD *buffer_len)
 {
   LPBYTE der_buffer= NULL;
-  DWORD der_buffer_length= 0;
+  DWORD der_buffer_length;
 
   if (buffer_len == NULL || *buffer_len == 0)
     return NULL;
@@ -191,7 +194,6 @@ end:
 }
 #endif
 
-#ifndef HAVE_GNUTLS
 static char *load_pub_key_file(const char *filename, int *pub_key_size)
 {
   FILE *fp= NULL;
@@ -207,15 +209,13 @@ static char *load_pub_key_file(const char *filename, int *pub_key_size)
   if (fseek(fp, 0, SEEK_END))
     goto end;
 
-  if ((*pub_key_size= ftell(fp)) < 0)
-    goto end;
-
+  *pub_key_size= ftell(fp);
   rewind(fp);
 
   if (!(buffer= malloc(*pub_key_size + 1)))
     goto end;
 
-  if (fread(buffer, *pub_key_size, 1, fp) != (size_t)*pub_key_size)
+  if (!fread(buffer, *pub_key_size, 1, fp))
     goto end;
 
   error= 0;
@@ -230,7 +230,7 @@ end:
   }
   return buffer;
 }
-#endif
+
 
 static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
 {
@@ -239,11 +239,10 @@ static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
   int rc= CR_ERROR;
 #if !defined(HAVE_GNUTLS)
   char passwd[MAX_PW_LEN];
-#ifdef HAVE_OPENSSL
-  unsigned char *rsa_enc_pw= NULL;
-  size_t rsa_size= 0;
-#else
   unsigned char rsa_enc_pw[MAX_PW_LEN];
+#ifdef HAVE_OPENSSL
+  int rsa_size;
+#else
   ULONG rsa_size;
 #endif
   unsigned int pwlen, i;
@@ -252,11 +251,9 @@ static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
   unsigned char buf[MA_SHA256_HASH_SIZE];
 
 #if defined(HAVE_OPENSSL)
-  EVP_PKEY *pubkey= NULL;
-  EVP_PKEY_CTX *ctx= NULL;
+  RSA *pubkey= NULL;
   BIO *bio;
-  size_t outlen;
-#elif defined(HAVE_WINCRYPT)
+#elif defined(HAVE_SCHANNEL)
   BCRYPT_KEY_HANDLE pubkey= 0;
   BCRYPT_OAEP_PADDING_INFO paddingInfo;
   LPBYTE der_buffer= NULL;
@@ -275,6 +272,14 @@ static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
   memmove(mysql->scramble_buff, packet, SCRAMBLE_LENGTH);
   mysql->scramble_buff[SCRAMBLE_LENGTH]= 0;
 
+  /* if a tls session is active we need to send plain password */
+  if (mysql->client_flag & CLIENT_SSL)
+  {
+    if (vio->write_packet(vio, (unsigned char *)mysql->passwd, (int)strlen(mysql->passwd) + 1))
+      return CR_ERROR;
+    return CR_OK;
+  }
+
   /* send empty packet if no password was provided */
   if (!mysql->passwd || !mysql->passwd[0])
   {
@@ -312,7 +317,7 @@ static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
   {
 #if defined(HAVE_GNUTLS)
      mysql->methods->set_error(mysql, CR_AUTH_PLUGIN_ERR, "HY000", 
-                               "RSA Encryption not supported - caching_sha2_password plugin was built with GnuTLS support");
+                               "RSA Encrytion not supported - caching_sha2_password plugin was built with GnuTLS support");
      return CR_ERROR;
 #else
     /* read public key file (if specified) */
@@ -338,19 +343,11 @@ static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
 #if defined(HAVE_OPENSSL)
     bio= BIO_new_mem_buf(filebuffer ? (unsigned char *)filebuffer : packet,
                          packet_length);
-    if (!(pubkey= PEM_read_bio_PUBKEY(bio, NULL, NULL, NULL)))
-      goto error;
-    if (!(ctx= EVP_PKEY_CTX_new(pubkey, NULL)))
-      goto error;
-    if (EVP_PKEY_encrypt_init(ctx) <= 0)
-      goto error;
-    if (EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_OAEP_PADDING) <= 0)
-      goto error;
-    rsa_size= EVP_PKEY_size(pubkey);
+    if ((pubkey= PEM_read_bio_RSA_PUBKEY(bio, NULL, NULL, NULL)))
+      rsa_size= RSA_size(pubkey);
     BIO_free(bio);
-    bio= NULL;
     ERR_clear_error();
-#elif defined(HAVE_WINCRYPT)
+#elif defined(HAVE_SCHANNEL)
     der_buffer_len= packet_length;
     /* Load pem and convert it to binary object. New length will be returned
        in der_buffer_len */
@@ -386,13 +383,9 @@ static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
 
     /* encrypt scrambled password */
 #if defined(HAVE_OPENSSL)
-    if (EVP_PKEY_encrypt(ctx, NULL, &outlen, (unsigned char *)passwd, pwlen) <= 0)
+    if (RSA_public_encrypt(pwlen, (unsigned char *)passwd, rsa_enc_pw, pubkey, RSA_PKCS1_OAEP_PADDING) < 0)
       goto error;
-    if (!(rsa_enc_pw= malloc(outlen)))
-      goto error;
-    if (EVP_PKEY_encrypt(ctx, rsa_enc_pw, &outlen, (unsigned char *)passwd, pwlen) <= 0)
-      goto error;
-#elif defined(HAVE_WINCRYPT)
+#elif defined(HAVE_SCHANNEL)
     ZeroMemory(&paddingInfo, sizeof(paddingInfo));
     paddingInfo.pszAlgId = BCRYPT_SHA1_ALGORITHM;
     if ((rc= BCryptEncrypt(pubkey, (PUCHAR)passwd, pwlen, &paddingInfo, NULL, 0, rsa_enc_pw,
@@ -416,14 +409,8 @@ static int auth_caching_sha2_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
 error:
 #if defined(HAVE_OPENSSL)
   if (pubkey)
-    EVP_PKEY_free(pubkey);
-  if (rsa_enc_pw)
-    free(rsa_enc_pw);
-  if (bio)
-    BIO_free(bio);
-  if (ctx)
-    EVP_PKEY_CTX_free(ctx);
-#elif defined(HAVE_WINCRYPT)
+    RSA_free(pubkey);
+#elif defined(HAVE_SCHANNEL)
   if (pubkey)
     BCryptDestroyKey(pubkey);
   if (publicKeyInfo)
@@ -460,7 +447,7 @@ static int auth_caching_sha2_init(char *unused1 __attribute__((unused)),
     int unused3     __attribute__((unused)),
     va_list unused4 __attribute__((unused)))
 {
-#if defined(HAVE_WINCRYPT)
+#if defined(HAVE_SCHANNEL)
   BCryptOpenAlgorithmProvider(&Sha256Prov, BCRYPT_SHA256_ALGORITHM, NULL, 0);
   BCryptOpenAlgorithmProvider(&RsaProv, BCRYPT_RSA_ALGORITHM, NULL, 0);
 #endif
@@ -469,9 +456,9 @@ static int auth_caching_sha2_init(char *unused1 __attribute__((unused)),
 /* }}} */
 
 /* {{{ auth_caching_sha2_deinit */
-static int auth_caching_sha2_deinit(void)
+static int auth_caching_sha2_deinit()
 {
-#if defined(HAVE_WINCRYPT)
+#if defined(HAVE_SCHANNEL)
   BCryptCloseAlgorithmProvider(Sha256Prov, 0);
   BCryptCloseAlgorithmProvider(RsaProv, 0);
 #endif
