@@ -1,6 +1,6 @@
 #!/bin/bash -ue
 # Copyright (C) 2013 Percona Inc
-# Copyright (C) 2017-2020 MariaDB
+# Copyright (C) 2017-2019 MariaDB
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; see the file COPYING. If not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston
-# MA  02110-1335  USA.
+# MA  02110-1301  USA.
 
 # Documentation:
 # http://www.percona.com/doc/percona-xtradb-cluster/manual/xtrabackup_sst.html
@@ -60,7 +60,7 @@ rebuild=0
 rebuildcmd=""
 payload=0
 pvformat="-F '%N => Rate:%r Avg:%a Elapsed:%t %e Bytes: %b %p' "
-pvopts="-f -i 10 -N $WSREP_SST_OPT_ROLE "
+pvopts="-f  -i 10 -N $WSREP_SST_OPT_ROLE "
 STATDIR=""
 uextra=0
 disver=""
@@ -83,14 +83,7 @@ fi
 pcmd="pv $pvopts"
 declare -a RC
 
-set +e
 INNOBACKUPEX_BIN=$(which mariabackup)
-if test -z $INNOBACKUPEX_BIN
-then
-  wsrep_log_error 'mariabackup binary not found in $PATH'
-  exit 42
-fi
-set -e
 XBSTREAM_BIN=mbstream
 XBCRYPT_BIN=xbcrypt # Not available in MariaBackup
 
@@ -137,7 +130,7 @@ get_keys()
 
     if [[ $encrypt -eq 0 ]];then 
         if $MY_PRINT_DEFAULTS xtrabackup | grep -q encrypt;then
-            wsrep_log_error "Unexpected option combination. SST may fail. Refer to http://www.percona.com/doc/percona-xtradb-cluster/manual/xtrabackup_sst.html"
+            wsrep_log_error "Unexpected option combination. SST may fail. Refer to http://www.percona.com/doc/percona-xtradb-cluster/manual/xtrabackup_sst.html "
         fi
         return
     fi
@@ -188,48 +181,23 @@ get_transfer()
             if nc -h 2>&1 | grep -q ncat;then 
                 # Ncat
                 tcmd="nc -l ${TSST_PORT}"
-            elif nc -h 2>&1 | grep -qw -- '-d\>';then
+            elif nc -h 2>&1 | grep -q -- '-d\>';then
                 # Debian netcat
-                if [ $WSREP_SST_OPT_HOST_IPv6 -eq 1 ];then
-                    # When host is not explicitly specified (when only the port
-                    # is specified) netcat can only bind to an IPv4 address if
-                    # the "-6" option is not explicitly specified:
-                    tcmd="nc -dl -6 ${TSST_PORT}"
-                else
-                    tcmd="nc -dl ${TSST_PORT}"
-                fi
+                tcmd="nc -dl ${TSST_PORT}"
             else
                 # traditional netcat
                 tcmd="nc -l -p ${TSST_PORT}"
             fi
         else
-            # Check to see if netcat supports the '-N' flag.
-            # -N Shutdown the network socket after EOF on stdin
-            # If it supports the '-N' flag, then we need to use the '-N'
-            # flag, otherwise the transfer will stay open after the file
-            # transfer and cause the command to timeout.
-            # Older versions of netcat did not need this flag and will
-            # return an error if the flag is used.
-            #
-            tcmd_extra=""
-            if nc -h 2>&1 | grep -qw -- -N; then
-                tcmd_extra+="-N"
-		wsrep_log_info "Using nc -N"
-            fi
-
-            # netcat doesn't understand [] around IPv6 address
             if nc -h 2>&1 | grep -q ncat;then
                 # Ncat
-                wsrep_log_info "Using Ncat as streamer"
-                tcmd="nc ${tcmd_extra} ${WSREP_SST_OPT_HOST_UNESCAPED} ${TSST_PORT}"
-            elif nc -h 2>&1 | grep -qw -- '-d\>';then
+                tcmd="nc ${REMOTEIP} ${TSST_PORT}"
+            elif nc -h 2>&1 | grep -q -- '-d\>';then
                 # Debian netcat
-                wsrep_log_info "Using Debian netcat as streamer"
-                tcmd="nc ${tcmd_extra} ${WSREP_SST_OPT_HOST_UNESCAPED} ${TSST_PORT}"
+                tcmd="nc ${REMOTEIP} ${TSST_PORT}"
             else
                 # traditional netcat
-                wsrep_log_info "Using traditional netcat as streamer"
-                tcmd="nc -q0 ${tcmd_extra} ${WSREP_SST_OPT_HOST_UNESCAPED} ${TSST_PORT}"
+                tcmd="nc -q0 ${REMOTEIP} ${TSST_PORT}"
             fi
         fi
     else
@@ -383,7 +351,7 @@ read_cnf()
     iopts=$(parse_cnf sst inno-backup-opts "")
     iapts=$(parse_cnf sst inno-apply-opts "")
     impts=$(parse_cnf sst inno-move-opts "")
-    stimeout=$(parse_cnf sst sst-initial-timeout 300)
+    stimeout=$(parse_cnf sst sst-initial-timeout 100)
     ssyslog=$(parse_cnf sst sst-syslog 0)
     ssystag=$(parse_cnf mysqld_safe syslog-tag "${SST_SYSLOG_TAG:-}")
     ssystag+="-"
@@ -410,8 +378,8 @@ read_cnf()
 
 get_stream()
 {
-    if [[ $sfmt == 'mbstream' || $sfmt == 'xbstream' ]];then
-        wsrep_log_info "Streaming with ${sfmt}"
+    if [[ $sfmt == 'xbstream' ]];then 
+        wsrep_log_info "Streaming with xbstream"
         if [[ "$WSREP_SST_OPT_ROLE"  == "joiner" ]];then
             strmcmd="${XBSTREAM_BIN} -x"
         else
@@ -496,7 +464,7 @@ cleanup_donor()
     if [[ -n ${XTRABACKUP_PID:-} ]];then 
         if check_pid $XTRABACKUP_PID
         then
-            wsrep_log_error "xtrabackup process is still running. Killing..."
+            wsrep_log_error "xtrabackup process is still running. Killing... "
             kill_xtrabackup
         fi
 
@@ -547,11 +515,25 @@ kill_xtrabackup()
 
 setup_ports()
 {
-    SST_PORT=${WSREP_SST_OPT_ADDR_PORT}
     if [[ "$WSREP_SST_OPT_ROLE"  == "donor" ]];then
-        REMOTEIP=${WSREP_SST_OPT_HOST}
-        lsn=${WSREP_SST_OPT_LSN}
-        sst_ver=${WSREP_SST_OPT_SST_VER}
+        if [ "${WSREP_SST_OPT_ADDR#\[}" != "$WSREP_SST_OPT_ADDR" ]; then
+            remain=$(echo $WSREP_SST_OPT_ADDR | awk -F '\\][:/]' '{ print $2 }')
+            REMOTEIP=$(echo $WSREP_SST_OPT_ADDR | awk -F '\\]:' '{ print $1 }')"]"
+            SST_PORT=$(echo $remain | awk -F '[:/]' '{ print $1 }')
+            lsn=$(echo $remain | awk -F '[:/]' '{ print $3 }')
+            sst_ver=$(echo $remain | awk -F '[:/]' '{ print $4 }')
+        else
+            SST_PORT=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $2 }')
+            REMOTEIP=$(echo $WSREP_SST_OPT_ADDR | awk -F ':' '{ print $1 }')
+            lsn=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $4 }')
+            sst_ver=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $5 }')
+        fi
+    else
+        if [ "${WSREP_SST_OPT_ADDR#\[}" != "$WSREP_SST_OPT_ADDR" ]; then
+            SST_PORT=$(echo ${WSREP_SST_OPT_ADDR} | awk -F '\\]:' '{ print $2 }')
+        else
+            SST_PORT=$(echo ${WSREP_SST_OPT_ADDR} | awk -F ':' '{ print $2 }')
+        fi
     fi
 }
 
@@ -584,7 +566,7 @@ check_extra()
                 # Xtrabackup works only locally.
                 # Hence, setting host to 127.0.0.1 unconditionally. 
                 wsrep_log_info "SST through extra_port $eport"
-                INNOEXTRA+=" --host=127.0.0.1 --port=$eport"
+                INNOEXTRA+=" --host=127.0.0.1 --port=$eport "
                 use_socket=0
             else 
                 wsrep_log_error "Extra port $eport null, failing"
@@ -594,8 +576,8 @@ check_extra()
             wsrep_log_info "Thread pool not set, ignore the option use_extra"
         fi
     fi
-    if [[ $use_socket -eq 1 ]] && [[ -n "$WSREP_SST_OPT_SOCKET" ]];then
-        INNOEXTRA+=" --socket=$WSREP_SST_OPT_SOCKET"
+    if [[ $use_socket -eq 1 ]] && [[ -n "${WSREP_SST_OPT_SOCKET}" ]];then
+        INNOEXTRA+=" --socket=${WSREP_SST_OPT_SOCKET}"
     fi
 }
 
@@ -630,8 +612,7 @@ recv_joiner()
     popd 1>/dev/null 
 
     if [[ ${RC[0]} -eq 124 ]];then 
-        wsrep_log_error "Possible timeout in receiving first data from "
-	                "donor in gtid stage: exit codes: ${RC[@]}"
+        wsrep_log_error "Possible timeout in receving first data from donor in gtid stage"
         exit 32
     fi
 
@@ -711,25 +692,20 @@ if ${INNOBACKUPEX_BIN} /tmp --help 2>/dev/null | grep -q -- '--version-check'; t
     disver="--no-version-check"
 fi
 
-iopts+=" --databases-exclude=\"lost+found\""
-
 if [[ ${FORCE_FTWRL:-0} -eq 1 ]];then 
     wsrep_log_info "Forcing FTWRL due to environment variable FORCE_FTWRL equal to $FORCE_FTWRL"
-    iopts+=" --no-backup-locks"
+    iopts+=" --no-backup-locks "
 fi
 
-INNOEXTRA=$WSREP_SST_OPT_MYSQLD
+
+INNOEXTRA=""
 
 INNODB_DATA_HOME_DIR=${INNODB_DATA_HOME_DIR:-""}
 # Try to set INNODB_DATA_HOME_DIR from the command line:
 if [ ! -z "$INNODB_DATA_HOME_DIR_ARG" ]; then
     INNODB_DATA_HOME_DIR=$INNODB_DATA_HOME_DIR_ARG
 fi
-# if no command line arg and INNODB_DATA_HOME_DIR environment variable
-# is not set, try to get it from my.cnf:
-if [ -z "$INNODB_DATA_HOME_DIR" ]; then
-    INNODB_DATA_HOME_DIR=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE innodb-data-home-dir '')
-fi
+# if INNODB_DATA_HOME_DIR env. variable is not set, try to get it from my.cnf
 if [ -z "$INNODB_DATA_HOME_DIR" ]; then
     INNODB_DATA_HOME_DIR=$(parse_cnf --mysqld innodb-data-home-dir '')
 fi
@@ -765,8 +741,8 @@ if [[ $ssyslog -eq 1 ]];then
             logger  -p daemon.info -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@" 
         }
 
-        INNOAPPLY="${INNOBACKUPEX_BIN} --innobackupex $disver $iapts \$INNOEXTRA --apply-log \$rebuildcmd \${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-apply"
-        INNOMOVE="${INNOBACKUPEX_BIN} --innobackupex ${WSREP_SST_OPT_CONF} $disver $impts  --move-back --force-non-empty-directories \${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move"
+        INNOAPPLY="${INNOBACKUPEX_BIN} --innobackupex $disver $iapts \$INNOEXTRA --apply-log \$rebuildcmd \${DATA} 2>&1  | logger -p daemon.err -t ${ssystag}innobackupex-apply "
+        INNOMOVE="${INNOBACKUPEX_BIN} --innobackupex ${WSREP_SST_OPT_CONF} $disver $impts  --move-back --force-non-empty-directories \${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move "
         INNOBACKUP="${INNOBACKUPEX_BIN} --innobackupex ${WSREP_SST_OPT_CONF} $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt \$itmpdir 2> >(logger -p daemon.err -t ${ssystag}innobackupex-backup)"
     fi
 
@@ -850,11 +826,9 @@ then
             exit 93
         fi
 
-        if [[ -z $(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE tmpdir "") && \
-              -z $(parse_cnf --mysqld tmpdir "") && \
-              -z $(parse_cnf xtrabackup tmpdir "") ]]; then
+        if [[ -z $(parse_cnf --mysqld tmpdir "") && -z $(parse_cnf xtrabackup tmpdir "") ]];then 
             xtmpdir=$(mktemp -d)
-            tmpopts=" --tmpdir=$xtmpdir"
+            tmpopts=" --tmpdir=$xtmpdir "
             wsrep_log_info "Using $xtmpdir as xtrabackup temporary directory"
         fi
 
@@ -876,12 +850,13 @@ then
         get_keys
         if [[ $encrypt -eq 1 ]];then
             if [[ -n $ekey ]];then
-                INNOEXTRA+=" --encrypt=$ealgo --encrypt-key=$ekey"
+                INNOEXTRA+=" --encrypt=$ealgo --encrypt-key=$ekey "
             else 
-                INNOEXTRA+=" --encrypt=$ealgo --encrypt-key-file=$ekeyfile"
+                INNOEXTRA+=" --encrypt=$ealgo --encrypt-key-file=$ekeyfile "
             fi
         fi
 
+
         check_extra
 
         wsrep_log_info "Streaming GTID file before SST"
@@ -902,6 +877,7 @@ then
             tcmd=" $scomp | $tcmd "
         fi
 
+
         send_donor $DATA "${stagemsg}-gtid"
 
         tcmd="$ttcmd"
@@ -973,32 +949,8 @@ then
     [[ -n $SST_PROGRESS_FILE ]] && touch $SST_PROGRESS_FILE
 
     ib_home_dir=$INNODB_DATA_HOME_DIR
-
-    WSREP_LOG_DIR=${WSREP_LOG_DIR:-""}
-    # Try to set WSREP_LOG_DIR from the command line:
-    if [ ! -z "$INNODB_LOG_GROUP_HOME_ARG" ]; then
-        WSREP_LOG_DIR=$INNODB_LOG_GROUP_HOME_ARG
-    fi
-    # if no command line arg and WSREP_LOG_DIR is not set,
-    # try to get it from my.cnf:
-    if [ -z "$WSREP_LOG_DIR" ]; then
-        WSREP_LOG_DIR=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE innodb-log-group-home-dir '')
-    fi
-    if [ -z "$WSREP_LOG_DIR" ]; then
-        WSREP_LOG_DIR=$(parse_cnf --mysqld innodb-log-group-home-dir '')
-    fi
-
-    ib_log_dir=$WSREP_LOG_DIR
-
-    # Try to set ib_undo_dir from the command line:
-    ib_undo_dir=${INNODB_UNDO_DIR_ARG:-""}
-    # if no command line arg then try to get it from my.cnf:
-    if [ -z "$ib_undo_dir" ]; then
-        ib_undo_dir=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE innodb-undo-directory "")
-    fi
-    if [ -z "$ib_undo_dir" ]; then
-        ib_undo_dir=$(parse_cnf --mysqld innodb-undo-directory "")
-    fi
+    ib_log_dir=$(parse_cnf --mysqld innodb-log-group-home-dir "")
+    ib_undo_dir=$(parse_cnf --mysqld innodb-undo-directory "")
 
     stagemsg="Joiner-Recv"
 
@@ -1075,13 +1027,7 @@ then
             find $ib_home_dir $ib_log_dir $ib_undo_dir $DATA -mindepth 1 -prune -regex $cpat -o -exec rm -rfv {} 1>&2 \+
 	fi
 
-        tempdir=$LOG_BIN_ARG
-        if [ -z "$tempdir" ]; then
-            tempdir=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE log-bin "")
-        fi
-        if [ -z "$tempdir" ]; then
-            tempdir=$(parse_cnf --mysqld log-bin "")
-        fi
+        tempdir=$(parse_cnf --mysqld log-bin "")
         if [[ -n ${tempdir:-} ]];then
             binlog_dir=$(dirname $tempdir)
             binlog_file=$(basename $tempdir)
