@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2002, 2013, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2021, MariaDB Corporation.
+   Copyright (c) 2011, 2013, Monty Program Ab.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "mariadb.h"
 #include "sql_priv.h"
@@ -50,98 +50,6 @@ double my_double_round(double value, longlong dec, bool dec_unsigned,
 
 #define MAX_DIGITS_IN_DOUBLE MY_GCVT_MAX_FIELD_WIDTH
 
-int MBR::within(const MBR *mbr)
-{
-  /*
-    We have to take into account the 'dimension' of
-    the MBR, where the dimension of a single point is 0,
-    the dimesion of an vertical or horizontal line is 1,
-    and finally the dimension of the solid rectangle is 2.
-  */
-    
-  int dim1= dimension();
-  int dim2= mbr->dimension();
-
-  DBUG_ASSERT(dim1 >= 0 && dim1 <= 2 && dim2 >= 0 && dim2 <= 2);
-
-  /*
-    Either/both of the two operands can degrade to a point or a
-    horizontal/vertical line segment, and we have to treat such cases
-    separately.
-   */
-  switch (dim1)
-  {
-  case 0:
-    DBUG_ASSERT(xmin == xmax && ymin == ymax);
-    switch (dim2)
-    {
-    case 0:
-      DBUG_ASSERT(mbr->xmin == mbr->xmax && mbr->ymin == mbr->ymax);
-      return equals(mbr);
-      break;
-    case 1:
-      DBUG_ASSERT((mbr->xmin == mbr->xmax && mbr->ymin != mbr->ymax) ||
-                  (mbr->ymin == mbr->ymax && mbr->xmin != mbr->xmax));
-      return ((xmin > mbr->xmin && xmin < mbr->xmax && ymin == mbr->ymin) ||
-              (ymin > mbr->ymin && ymin < mbr->ymax && xmin == mbr->xmin));
-      break;
-    case 2:
-      DBUG_ASSERT(mbr->xmin != mbr->xmax && mbr->ymin != mbr->ymax);
-      return (xmin > mbr->xmin && xmax < mbr->xmax &&
-              ymin > mbr->ymin && ymax < mbr->ymax);
-      break;
-    }
-    break;
-  case 1:
-    DBUG_ASSERT((xmin == xmax && ymin != ymax) ||
-                (ymin == ymax && xmin != xmax));
-    switch (dim2)
-    {
-    case 0:
-      DBUG_ASSERT(mbr->xmin == mbr->xmax && mbr->ymin == mbr->ymax);
-      return 0;
-      break;
-    case 1:
-      DBUG_ASSERT((mbr->xmin == mbr->xmax && mbr->ymin != mbr->ymax) ||
-                  (mbr->ymin == mbr->ymax && mbr->xmin != mbr->xmax));
-      return ((xmin == xmax && mbr->xmin == mbr->xmax && mbr->xmin == xmin &&
-               mbr->ymin <= ymin && mbr->ymax >= ymax) ||
-              (ymin == ymax && mbr->ymin == mbr->ymax && mbr->ymin == ymin &&
-               mbr->xmin <= xmin && mbr->xmax >= xmax));
-      break;
-    case 2:
-      DBUG_ASSERT(mbr->xmin != mbr->xmax && mbr->ymin != mbr->ymax);
-      return ((xmin == xmax && xmin > mbr->xmin && xmax < mbr->xmax &&
-               ymin >= mbr->ymin && ymax <= mbr->ymax) ||
-              (ymin == ymax && ymin > mbr->ymin && ymax < mbr->ymax &&
-               xmin >= mbr->xmin && xmax <= mbr->xmax));
-      break;
-    }
-    break;
-  case 2:
-    DBUG_ASSERT(xmin != xmax && ymin != ymax);
-    switch (dim2)
-    {
-    case 0:
-    case 1:
-      return 0;
-      break;
-    case 2:
-      DBUG_ASSERT(mbr->xmin != mbr->xmax && mbr->ymin != mbr->ymax);
-      return ((mbr->xmin <= xmin) && (mbr->ymin <= ymin) &&
-              (mbr->xmax >= xmax) && (mbr->ymax >= ymax));
-      break;
-
-    }
-    break;
-  }
-
-  // Never reached.
-  DBUG_ASSERT(false);
-  return 0;
-}
-
-
 /***************************** Gis_class_info *******************************/
 
 String Geometry::bad_geometry_data("Bad object", &my_charset_bin);
@@ -539,11 +447,7 @@ Geometry *Geometry::create_from_json(Geometry_buffer *buffer,
             goto handle_geometry_key;
           feature_type_found= 1;
         }
-        else /* can't understand the type. */
-          break;
       }
-      else /* The "type" value can only be string. */
-        break;
     }
     else if (key_len == coord_keyname_len &&
              memcmp(key_buf, coord_keyname, coord_keyname_len) == 0)
@@ -560,8 +464,6 @@ Geometry *Geometry::create_from_json(Geometry_buffer *buffer,
         coord_start= je->value_begin;
         if (ci && ci != &geometrycollection_class)
           goto create_geom;
-        if (json_skip_level(je))
-          goto err_return;
       }
     }
     else if (key_len == geometries_keyname_len &&
@@ -611,8 +513,6 @@ Geometry *Geometry::create_from_json(Geometry_buffer *buffer,
         if (feature_type_found)
           goto handle_geometry_key;
       }
-      else
-        goto err_return;
     }
     else
     {
@@ -1039,132 +939,6 @@ const Geometry::Class_info *Gis_point::get_class_info() const
 }
 
 
-/**
-  Function to calculate haversine.
-  Taking as arguments Point and Multipoint geometries.
-  Multipoint geometry has to be single point only.
-  It is up to caller to ensure valid input.
-
-  @param    g      pointer to the Geometry
-  @param    r      sphere radius
-  @param    error  pointer describing the error in case of the boundary conditions
-
-  @return distance in case without error, it is caclulcated distance (non-negative),
-                   in case error exist, negative value.
-*/
-double Gis_point::calculate_haversine(const Geometry *g,
-                                      const double sphere_radius,
-                                      int *error)
-{
-  DBUG_ASSERT(sphere_radius > 0);
-  double x1r, x2r, y1r, y2r;
-
-  // This check is done only for optimization purposes where we know it will
-  // be one and only one point in Multipoint
-  if (g->get_class_info()->m_type_id == Geometry::wkb_multipoint)
-  {
-    const char point_size= 4 + WKB_HEADER_SIZE + POINT_DATA_SIZE+1; //1 for the type
-    char point_temp[point_size];
-    memset(point_temp+4, Geometry::wkb_point, 1);
-    memcpy(point_temp+5, static_cast<const Gis_multi_point *>(g)->get_data_ptr()+5, 4);
-    memcpy(point_temp+4+WKB_HEADER_SIZE, g->get_data_ptr()+4+WKB_HEADER_SIZE,
-           POINT_DATA_SIZE);
-    point_temp[point_size-1]= '\0';
-    Geometry_buffer gbuff;
-    Geometry *gg= Geometry::construct(&gbuff, point_temp, point_size-1);
-    if (!gg || static_cast<Gis_point *>(gg)->get_xy_radian(&x2r, &y2r))
-    {
-      *error= 2;
-      return -1;
-    }
-  }
-  else
-  {
-    if (static_cast<const Gis_point *>(g)->get_xy_radian(&x2r, &y2r))
-    {
-      *error= 2;
-      return -1;
-    }
-  }
-  if (this->get_xy_radian(&x1r, &y1r))
-  {
-    *error= 2;
-    return -1;
-  }
-  //
-  // Check boundary conditions: longitude[-180,180]
-  if (!((x2r >= -M_PI && x2r <= M_PI) && (x1r >= -M_PI && x1r <= M_PI)))
-  {
-    *error=1;
-    return -1;
-  }
-  // Check boundary conditions: latitude[-90,90]
-  if (!((y2r >= -M_PI/2 && y2r <= M_PI/2) && (y1r >= -M_PI/2 && y1r <= M_PI/2)))
-  {
-    *error=-1;
-    return -1;
-  }
-  double dlat= sin((y2r - y1r)/2)*sin((y2r - y1r)/2);
-  double dlong= sin((x2r - x1r)/2)*sin((x2r - x1r)/2);
-  return 2*sphere_radius*asin((sqrt(dlat + cos(y1r)*cos(y2r)*dlong)));
-}
-
-
-/**
-  Function that calculate spherical distance of Point from Multipoint geometries.
-  In case there is single point in Multipoint geometries calculate_haversine()
-  can handle such case. Otherwise, new geometry (Point) has to be constructed.
-
-  @param    g pointer to the Geometry
-  @param    r sphere radius
-  @param    result pointer to the result
-  @param    err    pointer to the error obtained from calculate_haversin()
-
-  @return state
-  @retval TRUE  failed
-  @retval FALSE success
-*/
-int Gis_point::spherical_distance_multipoints(Geometry *g, const double r,
-                                              double *result, int *err)
-{  
-  uint32 num_of_points2;
-    // To find the minimum radius it cannot be greater than Earth radius
-  double res= 6370986.0;
-  double temp_res= 0.0;
-  const uint32 len= 4 + WKB_HEADER_SIZE + POINT_DATA_SIZE + 1;
-  char s[len];
-  g->num_geometries(&num_of_points2);
-  DBUG_ASSERT(num_of_points2 >= 1);
-  if (num_of_points2 == 1)
-  {
-    *result= this->calculate_haversine(g, r, err);
-    return 0;
-  }
-  for (uint32 i=1; i <= num_of_points2; i++)
-  {
-    Geometry_buffer buff_temp;
-    Geometry *temp;
-    const char *pt_ptr= g->get_data_ptr()+
-      4+WKB_HEADER_SIZE*i + POINT_DATA_SIZE*(i-1);
-
-    // First 4 bytes are handled already, make sure to create a Point
-    memset(s + 4, Geometry::wkb_point, 1);
-    if (g->no_data(pt_ptr, POINT_DATA_SIZE))
-      return 1;
-
-    memcpy(s + 5, g->get_data_ptr() + 5, 4);
-    memcpy(s + 4 + WKB_HEADER_SIZE, pt_ptr, POINT_DATA_SIZE);
-    s[len-1]= '\0';
-    temp= Geometry::construct(&buff_temp, s, len);
-    if (!temp)
-      return 1;
-    temp_res= this->calculate_haversine(temp, r, err);
-    if (res > temp_res)
-      res= temp_res;
-  }
-  *result= res;
-  return 0;
-}
 /***************************** LineString *******************************/
 
 uint32 Gis_line_string::get_data_size() const 
@@ -2296,89 +2070,6 @@ const Geometry::Class_info *Gis_multi_point::get_class_info() const
 }
 
 
-/**
-  Function that calculate spherical distance of Multipoints geometries.
-  In case there is single point in Multipoint geometries calculate_haversine()
-  can handle such case. Otherwise, new geometry (Point) has to be constructed.
-
-  @param    g pointer to the Geometry
-  @param    r sphere radius
-  @param    result pointer to the result
-  @param    err    pointer to the error obtained from calculate_haversin()
-
-  @return state
-  @retval TRUE  failed
-  @retval FALSE success
-*/
-int Gis_multi_point::spherical_distance_multipoints(Geometry *g, const double r,
-                                                    double *result, int *err)
-{
-  const uint32 len= 4 + WKB_HEADER_SIZE + POINT_DATA_SIZE + 1;
-  // Check how many points are stored in Multipoints
-  uint32 num_of_points1, num_of_points2;
-  // To find the minimum radius it cannot be greater than Earth radius
-  double res= 6370986.0;
-
-  /* From Item_func_sphere_distance::spherical_distance_points,
-     we are sure that there will be multiple points and we have to construct
-     Point geometry and return the smallest result.
-  */
-  num_geometries(&num_of_points1);
-  DBUG_ASSERT(num_of_points1 >= 1);
-  g->num_geometries(&num_of_points2);
-  DBUG_ASSERT(num_of_points2 >= 1);
-
-  for (uint32 i=1; i <= num_of_points1; i++)
-  {
-    Geometry_buffer buff_temp;
-    Geometry *temp;
-    double temp_res= 0.0;
-    char s[len];
-    const char *pt_ptr= get_data_ptr()+
-      4+WKB_HEADER_SIZE*i + POINT_DATA_SIZE*(i-1);
-    // First 4 bytes are handled already, make sure to create a Point
-    memset(s + 4, Geometry::wkb_point, 1);
-    if (no_data(pt_ptr, POINT_DATA_SIZE))
-      return 1;
-    memcpy(s + 5, this->get_data_ptr() + 5, 4);
-    memcpy(s + 4 + WKB_HEADER_SIZE, pt_ptr, POINT_DATA_SIZE);
-    s[len-1]= '\0';
-    temp= Geometry::construct(&buff_temp, s, len);
-    if (!temp)
-      return 1;
-    // Optimization for single Multipoint
-    if (num_of_points2 == 1)
-    {
-      *result= static_cast<Gis_point *>(temp)->calculate_haversine(g, r, err);
-      return 0;
-    }
-    for (uint32 j=1; j<= num_of_points2; j++)
-    {
-      Geometry_buffer buff_temp2;
-      Geometry *temp2;
-      char s2[len];
-      const char *pt_ptr= g->get_data_ptr()+
-        4+WKB_HEADER_SIZE*j + POINT_DATA_SIZE*(j-1);
-      // First 4 bytes are handled already, make sure to create a Point
-      memset(s2 + 4, Geometry::wkb_point, 1);
-      if (g->no_data(pt_ptr, POINT_DATA_SIZE))
-        return 1;
-      memcpy(s2 + 5, g->get_data_ptr() + 5, 4);
-      memcpy(s2 + 4 + WKB_HEADER_SIZE, pt_ptr, POINT_DATA_SIZE);
-      s2[len-1]= '\0';
-      temp2= Geometry::construct(&buff_temp2, s2, len);
-      if (!temp2)
-        return 1;
-      temp_res= static_cast<Gis_point *>(temp)->calculate_haversine(temp2, r, err);
-      if (res > temp_res)
-        res= temp_res;
-    }
-  }
-  *result= res;
-  return 0;
-}
-
-
 /***************************** MultiLineString *******************************/
 
 uint32 Gis_multi_line_string::get_data_size() const 
