@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA. */
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 #include "mariadb.h"
 #include "wsrep_binlog.h"
@@ -380,6 +380,20 @@ int wsrep_binlog_close_connection(THD* thd)
   DBUG_RETURN(0);
 }
 
+int wsrep_binlog_savepoint_set(THD *thd,  void *sv)
+{
+  if (!wsrep_emulate_bin_log) return 0;
+  int rcode = binlog_hton->savepoint_set(binlog_hton, thd, sv);
+  return rcode;
+}
+
+int wsrep_binlog_savepoint_rollback(THD *thd, void *sv)
+{
+  if (!wsrep_emulate_bin_log) return 0;
+  int rcode = binlog_hton->savepoint_rollback(binlog_hton, thd, sv);
+  return rcode;
+}
+
 #if 0
 void wsrep_dump_rbr_direct(THD* thd, IO_CACHE* cache)
 {
