@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
 
 #include "wsrep_var.h"
 
@@ -28,6 +28,8 @@
 
 ulong   wsrep_reject_queries;
 
+static long wsrep_prev_slave_threads = wsrep_slave_threads;
+
 int wsrep_init_vars()
 {
   wsrep_provider        = my_strdup(WSREP_NONE, MYF(MY_WME));
@@ -70,12 +72,6 @@ bool wsrep_on_check(sys_var *self, THD* thd, set_var* var)
 	    " innodb_lock_schedule_algorithm=FCFS and restart.", MYF(0));
     return true;
   }
-  if (new_wsrep_on && !WSREP_PROVIDER_EXISTS)
-  {
-    my_message(ER_WRONG_ARGUMENTS, "WSREP (galera) can't be enabled "
-               "if the wsrep_provider is unset or set to 'none'", MYF(0));
-    return true;
-  }
   return false;
 }
 
@@ -506,8 +502,6 @@ bool wsrep_cluster_address_update (sys_var *self, THD* thd, enum_var_type type)
   if (wsrep_start_replication())
   {
     wsrep_create_rollbacker();
-    WSREP_DEBUG("Cluster address update creating %ld applier threads running %lu",
-	    wsrep_slave_threads, wsrep_running_applier_threads);
     wsrep_create_appliers(wsrep_slave_threads);
   }
 
@@ -601,29 +595,21 @@ void wsrep_node_address_init (const char* value)
 
 static void wsrep_slave_count_change_update ()
 {
-  wsrep_slave_count_change = (wsrep_slave_threads - wsrep_running_applier_threads);
-  WSREP_DEBUG("Change on slave threads: New %ld old %lu difference %d",
-	  wsrep_slave_threads, wsrep_running_applier_threads, wsrep_slave_count_change);
+  wsrep_slave_count_change = (wsrep_slave_threads - wsrep_prev_slave_threads);
+  WSREP_DEBUG("Change on slave threads: New %lu old %lu difference %d",
+	  wsrep_slave_threads, wsrep_prev_slave_threads, wsrep_slave_count_change);
+  wsrep_prev_slave_threads = wsrep_slave_threads;
 }
 
 bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
 {
-  mysql_mutex_lock(&LOCK_thread_count);
-  bool res= false;
-
   wsrep_slave_count_change_update();
-
   if (wsrep_slave_count_change > 0)
   {
-    WSREP_DEBUG("Creating %d applier threads, total %ld", wsrep_slave_count_change, wsrep_slave_threads);
-    res= wsrep_create_appliers(wsrep_slave_count_change, true);
-    WSREP_DEBUG("Running %lu applier threads", wsrep_running_applier_threads);
+    wsrep_create_appliers(wsrep_slave_count_change);
     wsrep_slave_count_change = 0;
   }
-
-  mysql_mutex_unlock(&LOCK_thread_count);
-
-  return res;
+  return false;
 }
 
 bool wsrep_desync_check (sys_var *self, THD* thd, set_var* var)
@@ -720,9 +706,7 @@ static SHOW_VAR wsrep_status_vars[]=
   {"provider_name",     (char*) &wsrep_provider_name,     SHOW_CHAR_PTR},
   {"provider_version",  (char*) &wsrep_provider_version,  SHOW_CHAR_PTR},
   {"provider_vendor",   (char*) &wsrep_provider_vendor,   SHOW_CHAR_PTR},
-  {"thread_count",      (char*) &wsrep_running_threads,   SHOW_LONG_NOFLUSH},
-  {"applier_thread_count", (char*)&wsrep_running_applier_threads, SHOW_LONG_NOFLUSH},
-  {"rollbacker_thread_count", (char *)&wsrep_running_rollbacker_threads, SHOW_LONG_NOFLUSH},
+  {"thread_count",      (char*) &wsrep_running_threads,   SHOW_LONG_NOFLUSH}
 };
 
 static int show_var_cmp(const void *var1, const void *var2)
@@ -730,8 +714,8 @@ static int show_var_cmp(const void *var1, const void *var2)
   return strcasecmp(((SHOW_VAR*)var1)->name, ((SHOW_VAR*)var2)->name);
 }
 
-int wsrep_show_status (THD *thd, SHOW_VAR *var, void *buff,
-                       system_status_var *, enum_var_type scope)
+int wsrep_show_status (THD *thd, SHOW_VAR *var, char *buff,
+                       enum enum_var_type scope)
 {
   uint i, maxi= SHOW_VAR_FUNC_BUFF_SIZE / sizeof(*var) - 1;
   SHOW_VAR *v= (SHOW_VAR *)buff;
