@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2010, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2015, 2020, MariaDB Corporation.
+Copyright (c) 2015, 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -159,26 +159,25 @@ row_merge_create_fts_sort_index(
 
 	return(new_index);
 }
-
-/** Initialize FTS parallel sort structures.
-@param[in]	trx		transaction
-@param[in,out]	dup		descriptor of FTS index being created
-@param[in]	new_table	table where indexes are created
-@param[in]	opt_doc_id_size	whether to use 4 bytes instead of 8 bytes
-				integer to store Doc ID during sort
-@param[in]	old_page_size	page size of the old table during alter
-@param[out]	psort		parallel sort info to be instantiated
-@param[out]	merge		parallel merge info to be instantiated
+/*********************************************************************//**
+Initialize FTS parallel sort structures.
 @return TRUE if all successful */
 ibool
 row_fts_psort_info_init(
-        trx_t*                  trx,
-        row_merge_dup_t*        dup,
-        const dict_table_t*     new_table,
-        ibool                   opt_doc_id_size,
-        const page_size_t       old_page_size,
-        fts_psort_t**           psort,
-        fts_psort_t**           merge)
+/*====================*/
+	trx_t*			trx,	/*!< in: transaction */
+	row_merge_dup_t*	dup,	/*!< in,own: descriptor of
+					FTS index being created */
+	const dict_table_t*	new_table,/*!< in: table on which indexes are
+					created */
+	ibool			opt_doc_id_size,
+					/*!< in: whether to use 4 bytes
+					instead of 8 bytes integer to
+					store Doc ID during sort */
+	fts_psort_t**		psort,	/*!< out: parallel sort info to be
+					instantiated */
+	fts_psort_t**		merge)	/*!< out: parallel merge info
+					to be instantiated */
 {
 	ulint			i;
 	ulint			j;
@@ -211,7 +210,6 @@ row_fts_psort_info_init(
 
 	common_info->dup = dup;
 	common_info->new_table = (dict_table_t*) new_table;
-	common_info->old_page_size = old_page_size;
 	common_info->trx = trx;
 	common_info->all_info = psort_info;
 	common_info->sort_event = os_event_create(0);
@@ -805,8 +803,7 @@ DECLARE_THREAD(fts_parallel_tokenization)(
 	block = psort_info->merge_block;
 	crypt_block = psort_info->crypt_block;
 
-	const page_size_t	old_page_size =
-			psort_info->psort_common->old_page_size;
+	const page_size_t&	page_size = dict_table_page_size(table);
 
 	row_merge_fts_get_next_doc_item(psort_info, &doc_item);
 
@@ -836,7 +833,7 @@ loop:
 				doc.text.f_str =
 					btr_copy_externally_stored_field(
 						&doc.text.f_len, data,
-						old_page_size, data_len, blob_heap);
+						page_size, data_len, blob_heap);
 			} else {
 				doc.text.f_str = data;
 				doc.text.f_len = data_len;
@@ -865,8 +862,7 @@ loop:
 
 		num_doc_processed++;
 
-		if (UNIV_UNLIKELY(fts_enable_diag_print)
-		    && num_doc_processed % 10000 == 1) {
+		if (fts_enable_diag_print && num_doc_processed % 10000 == 1) {
 			ib::info() << "Number of documents processed: "
 				<< num_doc_processed;
 #ifdef FTS_INTERNAL_DIAG_PRINT
@@ -904,7 +900,7 @@ loop:
 			goto func_exit;
 		}
 
-		MEM_UNDEFINED(block[t_ctx.buf_used], srv_sort_buf_size);
+		UNIV_MEM_INVALID(block[t_ctx.buf_used], srv_sort_buf_size);
 		buf[t_ctx.buf_used] = row_merge_buf_empty(buf[t_ctx.buf_used]);
 		mycount[t_ctx.buf_used] += t_ctx.rows_added[t_ctx.buf_used];
 		t_ctx.rows_added[t_ctx.buf_used] = 0;
@@ -998,14 +994,12 @@ exit:
 					goto func_exit;
 				}
 
-#ifdef HAVE_valgrind
-				MEM_UNDEFINED(block[i], srv_sort_buf_size);
+				UNIV_MEM_INVALID(block[i], srv_sort_buf_size);
 
 				if (crypt_block[i]) {
-					MEM_UNDEFINED(crypt_block[i],
-						      srv_sort_buf_size);
+					UNIV_MEM_INVALID(crypt_block[i],
+							 srv_sort_buf_size);
 				}
-#endif /* HAVE_valgrind */
 			}
 
 			buf[i] = row_merge_buf_empty(buf[i]);
@@ -1013,7 +1007,7 @@ exit:
 		}
 	}
 
-	if (UNIV_UNLIKELY(fts_enable_diag_print)) {
+	if (fts_enable_diag_print) {
 		DEBUG_FTS_SORT_PRINT("  InnoDB_FTS: start merge sort\n");
 	}
 
@@ -1044,7 +1038,7 @@ exit:
 	}
 
 func_exit:
-	if (UNIV_UNLIKELY(fts_enable_diag_print)) {
+	if (fts_enable_diag_print) {
 		DEBUG_FTS_SORT_PRINT("  InnoDB_FTS: complete merge sort\n");
 	}
 
@@ -1219,9 +1213,11 @@ row_merge_write_fts_word(
 
 		error = row_merge_write_fts_node(ins_ctx, &word->text, fts_node);
 
-		if (UNIV_UNLIKELY(error != DB_SUCCESS)) {
-			ib::error() << "Failed to write word to FTS auxiliary"
-				" index table, error " << error;
+		if (error != DB_SUCCESS) {
+			ib::error() << "Failed to write word "
+				<< word->text.f_str << " to FTS auxiliary"
+				" index table, error (" << ut_strerr(error)
+				<< ")";
 			ret = error;
 		}
 
@@ -1373,7 +1369,7 @@ row_fts_sel_tree_propagate(
 	ulint		propogated,	/*<! in: tree node propagated */
 	int*		sel_tree,	/*<! in: selection tree */
 	const mrec_t**	mrec,		/*<! in: sort record */
-	rec_offs**	offsets,	/*<! in: record offsets */
+	ulint**		offsets,	/*<! in: record offsets */
 	dict_index_t*	index)		/*<! in/out: FTS index */
 {
 	ulint	parent;
@@ -1423,7 +1419,7 @@ row_fts_sel_tree_update(
 	ulint		propagated,	/*<! in: node to propagate up */
 	ulint		height,		/*<! in: tree height */
 	const mrec_t**	mrec,		/*<! in: sort record */
-	rec_offs**	offsets,	/*<! in: record offsets */
+	ulint**		offsets,	/*<! in: record offsets */
 	dict_index_t*	index)		/*<! in: index dictionary */
 {
 	ulint	i;
@@ -1445,7 +1441,7 @@ row_fts_build_sel_tree_level(
 	int*		sel_tree,	/*<! in/out: selection tree */
 	ulint		level,		/*<! in: selection tree level */
 	const mrec_t**	mrec,		/*<! in: sort record */
-	rec_offs**	offsets,	/*<! in: record offsets */
+	ulint**		offsets,	/*<! in: record offsets */
 	dict_index_t*	index)		/*<! in: index dictionary */
 {
 	ulint	start;
@@ -1505,7 +1501,7 @@ row_fts_build_sel_tree(
 /*===================*/
 	int*		sel_tree,	/*<! in/out: selection tree */
 	const mrec_t**	mrec,		/*<! in: sort record */
-	rec_offs**	offsets,	/*<! in: record offsets */
+	ulint**		offsets,	/*<! in: record offsets */
 	dict_index_t*	index)		/*<! in: index dictionary */
 {
 	ulint	treelevel = 1;
@@ -1529,11 +1525,10 @@ row_fts_build_sel_tree(
 		sel_tree[i + start] = int(i);
 	}
 
-	i = treelevel;
-	do {
+	for (i = treelevel; --i; ) {
 		row_fts_build_sel_tree_level(
-			sel_tree, --i, mrec, offsets, index);
-	} while (i > 0);
+			sel_tree, i, mrec, offsets, index);
+	}
 
 	return(treelevel);
 }
@@ -1556,7 +1551,7 @@ row_fts_merge_insert(
 	mem_heap_t*		heap;
 	dberr_t			error = DB_SUCCESS;
 	ulint*			foffs;
-	rec_offs**		offsets;
+	ulint**			offsets;
 	fts_tokenizer_word_t	new_word;
 	ib_vector_t*		positions;
 	doc_id_t		last_doc_id;
@@ -1579,6 +1574,9 @@ row_fts_merge_insert(
 	dict_index_t*		aux_index;
 	trx_t*			trx;
 
+	ut_ad(index);
+	ut_ad(table);
+
 	/* We use the insert query graph as the dummy graph
 	needed in the row module call */
 
@@ -1595,7 +1593,7 @@ row_fts_merge_insert(
 		heap, sizeof (*b) * fts_sort_pll_degree);
 	foffs = (ulint*) mem_heap_alloc(
 		heap, sizeof(*foffs) * fts_sort_pll_degree);
-	offsets = (rec_offs**) mem_heap_alloc(
+	offsets = (ulint**) mem_heap_alloc(
 		heap, sizeof(*offsets) * fts_sort_pll_degree);
 	buf = (mrec_buf_t**) mem_heap_alloc(
 		heap, sizeof(*buf) * fts_sort_pll_degree);
@@ -1619,10 +1617,10 @@ row_fts_merge_insert(
 
 		num = 1 + REC_OFFS_HEADER_SIZE
 			+ dict_index_get_n_fields(index);
-		offsets[i] = static_cast<rec_offs*>(mem_heap_zalloc(
+		offsets[i] = static_cast<ulint*>(mem_heap_zalloc(
 			heap, num * sizeof *offsets[i]));
-		rec_offs_set_n_alloc(offsets[i], num);
-		rec_offs_set_n_fields(offsets[i], dict_index_get_n_fields(index));
+		offsets[i][0] = num;
+		offsets[i][1] = dict_index_get_n_fields(index);
 		block[i] = psort_info[i].merge_block[id];
 		crypt_block[i] = psort_info[i].crypt_block[id];
 		b[i] = psort_info[i].merge_block[id];
@@ -1635,7 +1633,7 @@ row_fts_merge_insert(
 		count_diag += psort_info[i].merge_file[id]->n_rec;
 	}
 
-	if (UNIV_UNLIKELY(fts_enable_diag_print)) {
+	if (fts_enable_diag_print) {
 		ib::info() << "InnoDB_FTS: to insert " << count_diag
 			<< " records";
 	}
@@ -1657,6 +1655,7 @@ row_fts_merge_insert(
 	fts_table.type = FTS_INDEX_TABLE;
 	fts_table.index_id = index->id;
 	fts_table.table_id = table->id;
+	fts_table.parent = index->table->name.m_name;
 	fts_table.table = index->table;
 	fts_table.suffix = fts_get_suffix(id);
 
@@ -1724,6 +1723,7 @@ row_fts_merge_insert(
 	corresponding FTS index auxiliary tables */
 	for (;;) {
 		dtuple_t*	dtuple;
+		ulint		n_ext;
 		int		min_rec = 0;
 
 		if (fts_sort_pll_degree <= 2) {
@@ -1766,7 +1766,7 @@ row_fts_merge_insert(
 		}
 
 		dtuple = row_rec_to_index_entry_low(
-			mrec[min_rec], index, offsets[min_rec],
+			mrec[min_rec], index, offsets[min_rec], &n_ext,
 			tuple_heap);
 
 		row_fts_insert_tuple(
@@ -1801,17 +1801,13 @@ exit:
 	error = ins_ctx.btr_bulk->finish(error);
 	UT_DELETE(ins_ctx.btr_bulk);
 
-	trx->free();
+	trx_free(trx);
 
 	mem_heap_free(heap);
 
-	if (UNIV_UNLIKELY(fts_enable_diag_print)) {
+	if (fts_enable_diag_print) {
 		ib::info() << "InnoDB_FTS: inserted " << count << " records";
 	}
 
-	if (psort_info[0].psort_common->trx->get_flush_observer()) {
-		row_merge_write_redo(aux_index);
-	}
-
 	return(error);
 }
